//
//  QHVCPlayer+Advance.h
//  QHVCPlayerKit
//
//  Created by yinchaoyu on 2017/5/24.
//  Copyright © 2017年 qihoo 360. All rights reserved.
//

#import <CoreMedia/CoreMedia.h>
#import "QHVCPlayer.h"
#import "QHVCPlayerEnum.h"

/**
 播放器advanceDelegate
 */
@protocol QHVCPlayerAdvanceDelegate <NSObject>

@optional

/**
 播放器实例句柄创建
 */
- (void)onPlayerHandlerCreated:(QHVCPlayer *_Nonnull)player;

/**
 播放器销毁
 */
- (void)onPlayerDestoryed:(QHVCPlayer *_Nonnull)player;

/**
 流量计算回调函数

 @param dvbps 播放器下行视频每秒字节数
 @param dabps 播放器下行音频每秒字节数
 @param dvfps 播放器下行视频每秒帧数
 @param dafps 播放器下行音频每秒帧数
 @param fps 当前fps
 @param bitrate 当前bitrate
 */
- (void)onPlayerNetStats:(long)dvbps
                   dabps:(long)dabps
                   dvfps:(long)dvfps
                   dafps:(long)dafps
                     fps:(long)fps
                 bitrate:(long)bitrate
                  player:(QHVCPlayer *_Nonnull)player;

/**
 编码方式

 @param decodingType 编码类型
 @param sn sheduled_sn_url
 @param errorCode errorCode
 */
- (void)onPlayerDecodingType:(QHVCPlayerDecodingType)decodingType sn:(NSString *_Nullable)sn errorCode:(int)errorCode player:(QHVCPlayer *_Nonnull)player;

/**
 直播预览指定次数结束回调
 */
- (void)onPlayerPreviewFinished:(QHVCPlayer *_Nonnull)player;

/**
 播放实时回调音频buffer
 @param sampleBuffer 音频buffer
 */
- (void)onPlayerAudioDataCallback:(CMSampleBufferRef _Nonnull)sampleBuffer;

/**
 播放实时回调视频buffer
 @param pixelBuffer 视频buffer
 */
- (void)onPlayerVideoDataCallback:(CVPixelBufferRef _Nonnull)pixelBuffer player:(QHVCPlayer *_Nonnull)player;

/**
 返回原始数据包（注意该接口必须同步处理数据）
 @param packet 数据包结构体
 */
- (void)onPlayerOutputPacket:(QHVCPacket)packet;

/**
 录制完成
*/
- (void)onPlayerRecodeCompleted:(QHVCPlayer *_Nonnull)player;

@end

@interface QHVCPlayer (Advance)

/**
 播放器扩展delegate
 */
@property (nonatomic, weak) _Null_unspecified id<QHVCPlayerAdvanceDelegate> playerAdvanceDelegate;

/**
 * 开始预调度
 
 * @param sn 资源标识
 * @param channelId channelID
 * @param userId 用户ID
 * @param uSign 鉴权签名
 * @return YES表示成功 NO表示失败
 */
+ (BOOL)prepareScheduling:(NSString * _Nonnull)sn
                channelId:(NSString * _Nullable)channelId
                   userId:(NSString * _Nullable)userId
                    uSign:(NSString * _Nonnull)uSign;

/**
 * 开始预调度,需要使用特定调度地址的情况
 * @param scheduleUrl 预调度地址 格式参考http://g2.live.360.cn/
 * @param sn 资源标识
 * @param channelId channelID
 * @param userId 用户ID
 * @param uSign 鉴权签名
 * @return YES表示成功 NO表示失败
 */
+ (BOOL)prepareScheduling:(NSString * _Nonnull)scheduleUrl
                       sn:(NSString * _Nonnull)sn
                channelId:(NSString * _Nullable)channelId
                   userId:(NSString * _Nullable)userId
                    uSign:(NSString * _Nonnull)uSign;

/**
 初始化播放器
 
 @param SN 要播放的SN
 @param userId 用户id
 @param uSign 验证信息
 @param optionsDict 播放器可选属性字典，可传nil，或者根据实际需要选择部分或者全部属性设置，支持的全部属性如下：
@
 {@"streamType":@"intValue",@"hardDecode":@"boolValue",@"position":@"longValue",@"mute":@"boolValue",@"forceP2p":@"boolValue",@"playMode":@"intValue"，@"audioSessionCategory"：AVAudioSessionCategory,@"audioSessionMode":AVAudioSessionMod,@"audioSessionOptions":@(AVAudioSessionCategoryOptions),@"setPreferredIOBufferDuration"：@"boolValue",@"outputPacket"：@"boolValue",@"inputStream"：@"boolValue",@"enableAnalyzeSei":@"boolValue",@"enableDropSeiOutDate":@"boolValue",@"httpHeader":@"NSDictionaryValue",@"productId":@"NSStringValue"}
 
 @return 播放器对象
 */
- (QHVCPlayer * _Nullable)initWithSN:(NSString * _Nonnull)SN
                           channelId:(NSString * _Nullable)channelId
                              userId:(NSString * _Nullable)userId
                               uSign:(NSString * _Nonnull)uSign
                            options:(NSDictionary * _Nullable)optionsDict;

/**
 打开播放器流量统计,调用时机说明：播放器准备完成后会回调QHVCPlayerDelegate中的onPrepared方法，在该方法中调用openNetStats

 @param intervalBySecond 统计计算周期，单位：秒
 @参见 onPlayerNetStats delegate
 */
- (void)openNetStats:(uint)intervalBySecond;

/**
 关闭播放器流量统计
 */
- (void)closeNetStats;

/**
 镜像模式开关，播放器模式状态是非镜像状态，若启用镜像模式，调用该接口

 @param bMirror 镜像：YES，否则：NO
 */
- (void)setMirror:(BOOL)bMirror;

/**
 连麦加入Group
 
 @param groupID group id
 */
- (void)addToGroup:(int)groupID;

/**
 * 通知GPS，时区信息,用于优化调度CDN，定期由业务调用（切换省市，距离较远时调用，不要频繁调用）
 * @param lon 经度
 * @param lat 纬度
 */
- (void)playerGPSZoneLonLat:(double)lon latitude:(double)lat;

/**
 画质增强开关（本地控制模式时生效，控制模式由云端控制）
 @param enable 开启：YES，关闭：NO
 */
- (void)enableImageEnhanced:(BOOL)enable;

/**
 * 设置视频画质增强filter属性
 * @param brightness 亮度  取值范围(-0.2f, 0.35f)
 * @param contrast 对比度 取值范围(0.9f, 1.6f)
 * @param saturation 饱和度 取值范围(0.15f, 2f)
 */
- (void)enhancedImageQuality:(float)brightness contrast:(float)contrast saturation:(float)saturation;

/// 是否开启畸变矫正
/// @param enable 是否开启
- (void)enableUndistort:(BOOL)enable;

/// 设置畸变矫正参数
/// @param type 算法类型
/// @param param 算法参数，例如@{@"paramType":@(QHVCPlayerUndistortParamType), @"paramValue":@(int)}
- (void)setUndistortType:(QHVCPlayerUndistortType)type param:(NSDictionary *_Nullable)param;

/**
 * 倍速播放
 *
 * @param rate 播放速度，取值1~n（建议n<=5）
 */
- (void)setPlaybackRate:(float)rate;

/**
 获取播放倍速

 @return 倍速值
 */
- (float)getPlaybackRate;

/**
 销毁音频模块(直播生效)
 */
- (void)destroyAudioModule;

/**
 重启音频模块(直播生效)
 */
- (void)reStartAudioModule;

/**
 设置音频数据回调(注意：只能在onPlayerPrepared之后调用)
 @param isOutput yes输出 no不输出
 */
- (void)setAudioDataOutput:(BOOL)isOutput;

/**
 设置输出视频buffer
 @param isOutput yes输出 no不输出
 @param isUseSoftware yes用cpu,no用gpu转换纹理yuv->bgra
 */
- (void)setVideoDataOutput:(BOOL)isOutput useSoftware:(BOOL)isUseSoftware;

/**
 渲染模式输出宽高比例保持短边可见
 */
- (void)renderModeOutAspectScaleKeepShortEdgeVisible:(BOOL)isAlwaysVisible;

/**
 播放器接收外部数据流（要在prepare之后调用才有效）
 @param type 数据流类型
 @param data 数据流
 @param size 数据流长度
 @param pts 显示时间戳
 @param dts 解码时间戳
 @param isKey 是否是关键帧
 @return 输入成功返回非负整数
 */
- (int)inputStream:(QHVC_PACKET_TYPE)type data:(uint8_t *_Nonnull)data size:(int)size pts:(int64_t)pts dts:(int64_t)dts isKey:(int)isKey streamCodecContext:(QHVCStreamCodecContext *_Nullable)ctx;

/**
 * 开始录制(注意：此接口与setAudioDataOutput冲突，先设置的生效)
 * 注意：不支持暂停时录制
 *
 * @param filePath 录像存储路径（确保有读写权限）
 * @param recorderFormat 存储格式
 * @param config   配置
 * @return yes:成功  no: 失败
 */
- (BOOL)startRecorder:(NSString *_Nonnull)filePath recorderFormat:(QHVCRecorderFormat)recorderFormat recordConfig:(QHVCRecordConfig *_Nullable)config;

/**
 * 结束录制
 *
 * @return yes:成功  no: 失败
 */
- (BOOL)stopRecorder;

/**
iot场景下改变进度操作
*/
- (void)iotSeek;

/**
 视频转gif

 @param inputPath 视频路径
 @param outputPath gif路径
 @param sampleInterval 画面取帧间隔
 @param callback 进度状态回调
 */
+ (void)createGifWithVideo:(NSString *_Nonnull)inputPath outPutPath:(NSString *_Nonnull)outputPath sampleInterval:(NSUInteger)sampleInterval callback:(void (^_Nonnull)(float progress, BOOL completed))callback;

/**
 * 获取播放器运行参数
 * @return QHVCStatus;
 */
- (QHVCStatus)getStatus;

/**
 * 设置播放器参数
 * @param json 包含播放参数的json字符串;
 */
- (void)setControlParameter:(NSString *_Nonnull)json;


@end
